#pragma once


#include <QAction>

#include <RobotAPI/libraries/aron/core/data/variant/All.h>
#include <RobotAPI/libraries/aron/core/type/variant/All.h>
#include <RobotAPI/libraries/aron/core/type/visitor/variant/VariantVisitor.h>

class QTreeWidget;
class QPoint;

namespace armarx
{

    class AronTreeWidgetItem;

    // Visitor on aron types. It creates a  context menu dependent on the type of the AronTreeWidgetItem.
    // Its only used for Lists and Dicts.
    class AronTreeWidgetContextMenuVisitor : public armarx::aron::type::ConstVariantVisitor
    {

        AronTreeWidgetItem* parentItem;
        QTreeWidget* contextMenuParent;
        const QPoint& pos;
        int index;

    public:
        AronTreeWidgetContextMenuVisitor() = delete;
        AronTreeWidgetContextMenuVisitor(AronTreeWidgetItem* i,
                                         const QPoint& pos,
                                         QTreeWidget* contextMenuParent,
                                         int x);

        void visitAronVariant(const aron::type::ObjectPtr&) final;
        void visitAronVariant(const aron::type::DictPtr&) final;
        void visitAronVariant(const aron::type::PairPtr&) final;
        void visitAronVariant(const aron::type::TuplePtr&) final;
        void visitAronVariant(const aron::type::ListPtr&) final;
        void visitAronVariant(const aron::type::NDArrayPtr&) final;
        void visitAronVariant(const aron::type::MatrixPtr&) final;
        void visitAronVariant(const aron::type::QuaternionPtr&) final;
        void visitAronVariant(const aron::type::ImagePtr&) final;
        void visitAronVariant(const aron::type::PointCloudPtr&) final;
        void visitAronVariant(const aron::type::IntEnumPtr&) final;
        void visitAronVariant(const aron::type::IntPtr&) final;
        void visitAronVariant(const aron::type::LongPtr&) final;
        void visitAronVariant(const aron::type::FloatPtr&) final;
        void visitAronVariant(const aron::type::DoublePtr&) final;
        void visitAronVariant(const aron::type::BoolPtr&) final;
        void visitAronVariant(const aron::type::StringPtr&) final;
        void visitUnknown(Input&) final;

        void showMenuAndExecute();

    private:
        std::list<QAction> actions;
        std::vector<std::function<void()>> action_callbacks;

        // Creates a remove option if the element is a direct child of a list or dict
        void addDeleteAction();
        void executeDelete();

        void addAddAction();
        void executeAddElement();
    };
} // namespace armarx
