#pragma once
#include <vector>

#include <QHBoxLayout>
#include <QLineEdit>
#include <QObject>
#include <QTreeWidgetItem>
#include <QVBoxLayout>

#include "RobotAPI/libraries/aron/core/type/variant/ndarray/Matrix.h"

#include "CustomWidget.h"

namespace armarx
{
    // Custom Widget used to display Matrices
    // MAX_ROWS_DISPLAY and MAX_COLS_DISPLAY are the maximal number of elements displayable
    // everything else will not be displayed. However, there can still be made changes to this data with setText()
    class EditMatrixWidget : public CustomWidget
    {
        Q_OBJECT
    public:
        EditMatrixWidget() = delete;
        EditMatrixWidget(long numRows,
                         long numCols,
                         aron::type::matrix::ElementType elemType,
                         QTreeWidgetItem* currentWidget);

        static EditMatrixWidget* DynamicCast(QWidget*);
        static EditMatrixWidget* DynamicCastAndCheck(QWidget*);

        // Sets the text on all visible rows and cols.
        void setText(long row, long col, const std::string& str);
        std::string getText(long row, long col);
        void highlightUnparsableEntries();
        bool hasParseErrors();
        // returns an empty vector if parsing failed. Else, contains the individual bytes.
        std::vector<unsigned char> parseElement(long row, long col);


    private:
        // Dimensions of the underlying Type to represent
        long realRows = 1;
        long realCols = 1;

        // maximum of rows / cols to display
        static constexpr long MAX_ROWS_DISPLAY = 5;
        static constexpr long MAX_COLS_DISPLAY = 5;
        long getDisplayedRows() const;
        long getDisplayedCols() const;

        // Want to add dots to show the user that not all is displayed. This needs some wrapping layouts
        QVBoxLayout* outerVerticalLayout;
        QHBoxLayout* innerHorizontalLayout;

        // Using row major layout
        std::vector<QLineEdit*> dispElements;
        // rows<cols<text>>: stores get and set messages for hidden elements (if there is not enough space for all elements)
        std::vector<std::vector<std::string>> hiddenElems;

        template <typename T>
        static std::vector<unsigned char> toByteVector(const std::string& str);

        // TODO: It would be nice to have this code somewhere else, this way, at least the GUI-responses
        // are restricted to this Widget (signal & slot stuff)
        aron::type::matrix::ElementType elemType;

    private slots:
        void matrixElementChanged();
    };

    template <typename T>
    std::vector<unsigned char>
    EditMatrixWidget::toByteVector(const std::string& str)
    {
        auto val = simox::alg::to_<T>(str);
        std::vector<unsigned char> res(sizeof(val), 0);
        *reinterpret_cast<T*>(res.data()) = val;
        return res;
    }
} // namespace armarx
