#pragma once

#include <vector>

#include <RobotAPI/interface/units/GraspCandidateProviderInterface.h>
#include <RobotAPI/libraries/armem/client.h>
#include <RobotAPI/libraries/armem/client/Reader.h>

namespace armarx::armem
{

    class GraspCandidateReader
    {
    public:
        GraspCandidateReader();

        void connect(armem::client::MemoryNameSystem& memoryNameSystem, bool use = true);

        ::armarx::grasping::GraspCandidatePtr
        queryGraspCandidateInstanceByID(armem::MemoryID const& id) const;

        ::armarx::grasping::GraspCandidateDict
        queryGraspCandidateInstancesByID(std::vector<armem::MemoryID> const& ids) const;

        ::armarx::grasping::BimanualGraspCandidatePtr
        queryBimanualGraspCandidateInstanceByID(armem::MemoryID const& id) const;

        ::armarx::grasping::GraspCandidateDict
        queryLatestGraspCandidateEntity(std::string const& provider,
                                        std::string const& entity) const;

        std::map<std::string, ::armarx::grasping::BimanualGraspCandidatePtr>
        queryLatestBimanualGraspCandidateEntity(std::string const& provider,
                                                std::string const& entity) const;

        ::armarx::grasping::GraspCandidateDict
        queryLatestGraspCandidates(std::string const& provider = "") const;

        ::armarx::grasping::GraspCandidateDict queryGraspCandidatesNewerThan(
            std::string const& provider = "",
            const armarx::DateTime& timestamp = armarx::DateTime::Now()) const;

        std::map<std::string, ::armarx::grasping::BimanualGraspCandidatePtr>
        queryLatestBimanualGraspCandidates(std::string const& provider = "") const;


        void registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& def);


    private:
        ::armarx::grasping::GraspCandidateDict
        getGraspCandidatesFromResultSet(armem::client::QueryResult const& qResult) const;

        std::map<std::string, ::armarx::grasping::BimanualGraspCandidatePtr>
        getBimanualGraspCandidatesFromResultSet(armem::client::QueryResult const& qResult) const;

        armem::client::Reader memoryReader;

        // Properties
        struct Properties
        {
            std::string memoryName = "Grasp";
            std::string graspCandidateMemoryName = "GraspCandidate";
            std::string bimanualGraspCandidateMemoryName = "BimanualGraspCandidate";
        } properties;

        const std::string propertyPrefix = "mem.grasping.";
    };

} // namespace armarx::armem
