#pragma once

#include <mutex>

#include <ArmarXCore/core/application/properties/forward_declarations.h>

#include <RobotAPI/interface/units/GraspCandidateProviderInterface.h>
#include <RobotAPI/libraries/armem/client.h>
#include <RobotAPI/libraries/armem/client/Writer.h>

namespace armarx::armem
{
    class GraspCandidateWriter
    {
    public:
        GraspCandidateWriter() = default;

        void connect(armem::client::MemoryNameSystem& memoryNameSystem);

        void registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& def);

        bool commitGraspCandidate(const armarx::grasping::GraspCandidate& candidate,
                                  const armem::Time& timestamp,
                                  const std::string& provider);
        bool commitBimanualGraspCandidate(const armarx::grasping::BimanualGraspCandidate& candidate,
                                          const armem::Time& timestamp,
                                          const std::string& provider);

        bool commitGraspCandidateSeq(const armarx::grasping::GraspCandidateSeq& candidates,
                                     const armem::Time& timestamp,
                                     const std::string& provider);
        bool commitBimanualGraspCandidateSeq(
            const armarx::grasping::BimanualGraspCandidateSeq& candidates,
            const armem::Time& timestamp,
            const std::string& provider);


    private:
        bool commitToMemory(const std::vector<armarx::aron::data::DictPtr>& instances,
                            const std::string& providerName,
                            const std::string& entityName,
                            const armem::Time& timestamp,
                            const std::string& coreMemoryName);

        armem::client::Writer memoryWriter;

        struct Properties
        {
            std::string memoryName = "Grasp";
            std::string graspMemoryName = "GraspCandidate";
            std::string bimanualGraspMemoryName = "BimanualGraspCandidate";
        } properties;

        std::mutex memoryWriterMutex;

        const std::string propertyPrefix = "mem.grasping.";
    };


} // namespace armarx::armem
