#pragma once

#include <functional>

// BaseClass
#include <RobotAPI/libraries/armem/core/MemoryID.h>
#include <RobotAPI/libraries/armem/core/wm/aron_conversions.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>

#include "MemoryItem.h"

namespace armarx::armem::server::ltm::detail
{
    /// @brief Interface functions for the longterm memory classes
    class EntityInstanceBase : public MemoryItem
    {
    public:
        struct Statistics
        {
            long recordedData = 0;
            long recordedMetaData = 0;
        };

    public:
        using MemoryItem::MemoryItem;

        /// return the full sub-ltm as a wm::EntitySnapshot with only references
        /// the ltm may be huge, use with caution
        void
        loadAllReferences(armem::wm::EntitySnapshot& e) const
        {
            _loadAllReferences(e);
        }

        /// convert the references of the input into a wm::Memory
        void
        resolve(armem::wm::EntityInstance& e) const
        {
            _resolve(e);
        }

        /// encode the content of a wm::Memory and store
        nlohmann::json
        store(const armem::wm::EntityInstance& e)
        {
            return _store(e);
        }

        /// statistics
        void
        resetStatistics()
        {
            statistics.recordedData = 0;
            statistics.recordedMetaData = 0;
        }

        Statistics
        getStatistics() const
        {
            return statistics;
        }

        static std::string getLevelName();

    protected:
        virtual void _loadAllReferences(armem::wm::EntitySnapshot&) const = 0;
        virtual void _resolve(armem::wm::EntityInstance&) const = 0;
        virtual nlohmann::json _store(const armem::wm::EntityInstance&) = 0;

    protected:
        mutable std::recursive_mutex ltm_mutex;

        Statistics statistics;
    };
} // namespace armarx::armem::server::ltm::detail
