/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <map>
#include <optional>

#include <RobotAPI/libraries/armem/client/util/SimpleReaderBase.h>
#include <RobotAPI/libraries/armem_objects/types.h>

namespace armarx::armem::obj::clazz
{
    class ClassReader : public armem::client::util::SimpleReaderBase
    {
    public:
        ClassReader() = default;

        std::optional<armem::clazz::ObjectClass> getObjectClass(const std::string& providerName,
                                                                const armarx::ObjectID& id);

        /**
         * @brief Get object class information for object class IDs.
         * @param objectIDs The object class IDs.
         * @return The corresponding object classes.
         * @throw armarx::armem::error::QueryFailed If the memory query failed.
         */
        std::map<armarx::ObjectID, armem::clazz::ObjectClass>
        getObjectClasses(const std::vector<armarx::ObjectID>& objectIDs);

    protected:
        std::string propertyPrefix() const final;
        Properties defaultProperties() const final;
    };


} // namespace armarx::armem::obj::clazz
