/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Christoph Pohl ( christoph dot pohl at kit dot edu )
 * @date       05.12.23
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/libraries/skills/provider/SkillFactory.h>

namespace armarx
{
    namespace skills
    {
//        template <typename T>
//        concept takesContext = requires(typename T::Context cntxt) {
//            { T::connectTo(cntxt) } -> void;
//        };


        template <typename SkillT>
        class SkillWithContextBlueprint : public SkillBlueprint
        {
        public:
            using ContextT = typename SkillT::Context;

            template <typename... Args>
            SkillWithContextBlueprint(Args&&... args) :
                SkillBlueprint(*SkillBlueprint::ForSkill<SkillT>(std::forward<Args>(args)...))
            {
            }

            virtual std::unique_ptr<Skill>
            createSkill(const ProviderID& pid) const override
            {
                if (not isConnected_)
                {
                    return nullptr;
                }
                auto s = _createSkill();
                s->setProviderId(pid);
                static_cast<SkillT*>(s.get())->connectTo(context_);
                return s;
            }

            virtual SkillDescription
            createSkillDescription(const skills::ProviderID& pid) const
            {
                auto s = _createSkill();
                s->setProviderId(pid);
                return s->getSkillDescription();
            }

            void
            connectTo(const ContextT& context)
            {
                context_ = context;
                isConnected_ = true;
            }

        private:
            std::atomic_bool isConnected_ = false;
            ContextT context_;
        };
    } // namespace skills
} // namespace armarx